/***
   NAME
     cr.c
   NOTES
     The problem specific definition file size and age structured krill model

     Terminology:
    HISTORY
    Alexey Ryabov - April 08, 2017: Last Revision.
***/
extern "C" {
#include  "escbox.h"
}

using namespace std;
#include "iniFile.h"
#include "SAM_Indexes.h"
#include "mymath.h"
#include <math.h>
#include <time.h>
#include <stdio.h>
#include <iostream>
#include <stdlib.h>
#include <fstream>
#include "SFMT.h"
#include "SFMT.c"



/*
 *===========================================================================
 *
 *		LABELLING ENVIRONMENT AND I-STATE VARIABLES
 *
 *===========================================================================
 */
//Environmental variables, time and resources
#define	time		(env[0]) //time
#define	chl 		(env[1]) //chlorophyll-a
#define	ice_alg		(env[2]) //ice-algae chlorophyll

//The i-state variables which characterize every cohort
#define	age		    (i_state(0))
#define	weight	    (i_state(1))

//Parameters of ID cards

#define	IDstate  	    (i_const( 0))  // 1 -- embryos, 2 -- larvae, 3 -- juveniles, 4 -- adults
#define	IDingest_chl	(i_const( 1))
#define	IDingest_ice	(i_const( 2))
#define	IDassim 	    (i_const( 3))
#define	IDmaint 	    (i_const( 4))
#define	IDnet_assim		(i_const( 5))
#define	IDgrowth		(i_const( 6))
#define	IDmetabrate	    (i_const( 7))
#define	IDfecundity	    (i_const( 8))
#define	IDmort          (i_const( 9))
#define	IDAffectChl     (i_const(10))

#define	CONS		0

#define	SMALL		0.001
#define	TINY		1.0E-6
#define MINFOOD		1.0E-12



/*
 *===========================================================================
 *
 *		DEFINING AND LABELLING CONSTANTS AND PARAMETERS
 *
 *===========================================================================
 */

#define integer int

/*
 *===========================================================================
 *
 *		DEFINING GLOBAL VARIABLES AND USER-IMPLEMENTED FUNCTIONS
 *
 *===========================================================================
 */


////----------declare model parameters--------------------------------------------
double Sigma1, Sigma2, Cw;         //Conversion from Weight to Length and back
double EggWeigth, EggSize;                // Egg size and weight
double L_repr_min, L_repr_mid, L_delta; // Parameters which define splitting of assimilated carbon between growth and reproduction

double Eps_assim; //assimilation efficiency mg DW/mg C consumed
double C2DW;   // [mg body Carbon/mg dry weight]  conversion of units
double Esp_ovary;  // the ratio eggs dry weight to ovary dry weigth
double Egg_hatch; // the number of eggs which survive


double Metabolic_rate_min;  //		;the lowest level of the metabolic rate for adults
double Metabolic_bestday;   //            ;0 ; the day of year when the metabolic rate is highest
double Metabolic_high_duration; //	;120 Duration of high metabolic period
double pi_Metab_duration_2; // (pi * Metabolic_high_duration/2)^2


//Ingestion rates of Larvae, Juveniles and Adults, for Larvae it is possible to
//have different ingestion rates for Chl-a and ice-algae ingestion
double I_max_0;        //20
double I_max_60;       //10
double I_T_alpha;       //0

double HS_l, HS_j, HS_a; //half-saturation constants for phytoplankton ingestion
double AffectChl_l, AffectChl_j, AffectChl_a; //1 if the cohort affects the chl coincentration, 0 otherwise
int Holling_type; //The type of the ingestion functional response. 2 Holling 2 = C/C+H, 3 -- Holling 3 = C^2/(C^2 + H^2)


double T_l, T_j, T_a; //The maintenance coefficient
double Const_growth=1; //[year^-1] if > 0 then krill increases in size with this growth rate. To test the model.


int    Use_Index;       //=1          ;  0 - no index, 1 SAM (monthly 1979-2015), K_Ph = K_0 * exp(SAM * Ind_regression)
double Ind_regression;  //=0.5   ;  the slope in the dependence above
int    Index_starts;    //=1979    ;  year when the index starts, this will be year Zero in the model results
int    Index_ends;      //=2015      ;  the last full year of the index.
double Index_curr(0);   //the current value of the index

double Chl_K;            //   [mg Chl/m^3]    Carrying capasity for Chl production
double Chl_K_pert;//=-1   ;   [mg C/m^3]      Carrying capasity, perturbed
double Chl_K_cur(0);      //current value of chla carring capasity
double Chl_infl; //=0.1         ;   [mg C/m^3/day]    Stationary inflow of chlorophyll
double Chl_infl_pert; //=-1;   [mg C/m^3/day]
double Chl_min;//=0.01    ;   [mg Chl/m^3]    Minimum (winter) level of Chl. Necessary only for numerical purpose
double Chl_g_max;//=1     ;   [1/day]         Maximal growht rate of Chl
double Chl_bestday;//=0   ;   [day]           The day of year, when the maximal production of chlrorofill is reached
double Chl_duration;//=50 ;   [day]           Duration of summer, when chlorophyll net production is positive
double pi_Chl_duration_2; // (pi * Chl_duration/2)^2
double Chl_loss;//=0.2    ;   [1/day]         Natural loss rate of Chl-a
double C2Chl;//=100       ;   [mg C/mg Chl-a] Carbon to Chlorophill ratio

int    Perturb_year; //;   [years]       if e.g. Perturb_year = 3 then each third year take perturbed paramters, e.g Chl_K_pert instead of Chl_K
double Chl_ice_K;//=50	      ;   [mg Chl/m^3]    Carrying capasity for ice algae
double Chl_ice_K_pert; //=-1  ;   [mg C/m^3]      Carrying capasity for ice algae, perturbed
double Chl_ice_min;//=-1      ;   [mg Chl/m^3]    Minimum (summer) level of ice algae, if -1 then take Chl_min
double Chl_ice_g_max;//=-1    ;   [1/day]         Maximal growht rate of Ice Algae if -1 then take Chl_g_max
double Chl_ice_bestday;//=200 ;   [day]           The day of year, when the maximal production of ice algae is reached
double Chl_ice_duration;//=20 ;   [days]          Duration of ice algae period
double pi_Chl_ice_duration_2; //         (pi * Chl_ice_duration/2)^2
double Chl_ice_loss;            //=-1     ;   [1/day]         Natural loss rate of ice algae, if -1 then use Chl_loss
double C2Chl_ice;//=100       ;   [mg C/mg Chl-a] Carbon to ice-algae-chlorophill ratio
double Water_col_ice_layer_ratio; //=20;   the ratio between the depth of the water column and of the ice algae layer


double Age_l;              //[days]   when Embryones start to eat.
double L_larv_juv, L_delta_l_j;            //[mm]   L_larv_juv=10   ;[mm]   the minimal size of subadult
double Age_j;              //[years]   more than one year old krill can shrink if size > L_larv_juv
double Age_a;               //[year]   two years old krill the same as juveniles and also can reproduce if large enough
double Age_max;                 //;[year]   maximal life span of an individual

double Mortality_e; //the mortality of  embryones
double Mortality_l; //mortality of larvae
double Mortality_starv_l; //starvation mortality of larvae
double Mortality_j; //the mortality of juvenails
double Mortality_a; //the mortality of adults
double Mort_factor;  //if zero then indifiduals do not die independend of the mortality level


double Use_allometric_mortality; //=1      ;1 if we use allometrix mortality accoring McGurk 1986
double Mortality_allom_w; //=1.92          ; [1/year]  1.92 = 5.26 * 0.001 * 365
double Mortality_allom_sigma; //=-0.25     ;
double Mortality_allom_max; //=10          ; [1/year]; 10 the maximal allowed mortality

double Distrib_output_intv;  //=0.1  ;   [year]          output interval of distributions
double Distrib_fec;     //=1       ;   1 to save fecandity distrib over length
double Distrib_bottleneck;     //=1       ;   1 to save (Sb beta(s, R) + g(s, R))/s - d(s, R)
double Distrib_outout_time(-1); //time of the last output of the distributions

double Distrib_length;  //=1    ;   1 to save length distribution
double Distrib_len_min;     //=0    ;   minimal length
double Distrib_len_max;     //=70   ;  [mm] maximal length
double Distrib_len_bins;    //=50;  number of bins

double Distrib_age;     //=1    ;   1 to save age distribution
double Distrib_age_min;     //=0    ;   minimal age
double Distrib_age_max;     //=7    ;  [mm] maximal age
double Distrib_age_bins;    //=50   ;  number of bins



/// ------------declare functions --------------------------------/////


static void		UpdateIDcards(double *env, population *pop, int setfec);


inline double W2L(double Weigth);
inline double L2W(double Length);

inline double fermi(double x, double alpha); //equals 1 for x< 0 and 1 for x > 0
inline double fch(double FromValue, double ToValue, double z, double AtValue, double Alpha);
inline double Split_K(double Length);
inline double SaveDiv(double a, double b);
inline double Max_ing_rate(double Weight);




/// -----------------------------------------------------------------------------------////




/*
 *===========================================================================
 *
 * USER INITIALIZATION ROUTINE ALLOWS OPERATIONS ON INITIAL POPULATIONS
 *
 *===========================================================================
 */


inline double SaveDiv(double a, double b){
    if ((b == 0) && (a == 0))
        return 0;
    else
        return a/b;
};

/// read a float number from ini-file
double GetFloat(CIniFile &iniFile, string const keyname, string const valuename, double const defValue = 0.0) {
    double res = iniFile.GetValueF(keyname, valuename, defValue);
    if (res ==-999) {
        cout << "   "<< keyname << "." << valuename << " was not found in ini file";
        throw;
    } else if (res ==-1){
        res = defValue;
        cout << "   "<< keyname << "." << valuename << " = " <<res  << " (Default value)" << endl;
    } else
        cout << "   "<< keyname << "." << valuename << " = " <<res  << endl;
    return res;
}


/// read an integer number from ini-file
int GetInt(CIniFile& iniFile, string const keyname, string const valuename, int const defValue = 0) {
    int res = iniFile.GetValueI(keyname, valuename, defValue);
    if (res ==-999) {
        cout << "   "<< keyname << "." << valuename << " was not found in ini file";
        throw;
    } else if (res ==-1){
        res = defValue;
        cout << "   "<< keyname << "." << valuename << " = " <<res  << " (Default value)" << endl;
    } else
        cout << "   "<< keyname << "." << valuename << " = " <<res  << endl;
    return res;
}


void	UserInit(int argc, char	**argv, double *env, population *pop)

{
    CIniFile iniFile("./params.ini");
    iniFile.ReadFile();

    cout << "Loading model paramters from params.ini" << endl;

    //Solver parameters

    cout << "Consumers:" << endl;
///Length <-> Dry weight conversion
    cout << "Length <-> Dry weight conversion" << endl;
    Cw = GetFloat(iniFile, "Consumers", "Cw", -999);
    Sigma1 = GetFloat(iniFile, "Consumers", "Sigma1", -999);
    Sigma2 = GetFloat(iniFile, "Consumers", "Sigma2", -999);



///Threshold ages
    cout << "Threshold ages" << endl;
    Age_l   =   GetFloat(iniFile, "Consumers", "Age_l", -999)/365;     //[year]   when Embryones start to eat.
    L_larv_juv =  GetFloat(iniFile, "Consumers", "L_larv_juv", -999);       //[mm]   Size_juvenile=10   ;[mm]   the minimal size of subadult
    L_delta_l_j =  GetFloat(iniFile, "Consumers", "L_delta_l_j", -999);       //[mm]   Size_juvenile=10   ;[mm]   the minimal size of subadult
    Age_j    =     GetFloat(iniFile, "Consumers", "Age_j", -999);          //[year]   one years old krill the same as juveniles and also can reproduce if large enough
    Age_a    =     GetFloat(iniFile, "Consumers", "Age_a", -999);          //[year]   two years old krill the same as juveniles and also can reproduce if large enough
    Age_max      =     GetFloat(iniFile, "Consumers", "Age_max", -999);                 //;[year]   maximal life span of an individual


/// Food assimilation
    cout << "Food assimilation " << endl;
    Eps_assim = GetFloat(iniFile, "Consumers", "Eps_assim", -999);//assimilation efficiency
    C2DW      = GetFloat(iniFile, "Consumers", "C2DW", -999); // [mg body Carbon/mg dry weight]  conversion of units

///Ingestion rates and half-saturation constant for adults.
    cout << "Ingestion rates" << endl;
    I_max_0 = GetFloat(iniFile, "Consumers", "I_max_0", -999);        //=20 %  body carbon/d
    I_max_0 = I_max_0/100 * C2DW * 365; // %body carbon /d ->  mg Carbon /(mg DW * year)
    I_max_60 = GetFloat(iniFile, "Consumers", "I_max_60", -999);        //=10 %  body carbon/d
    I_max_60 = I_max_60/100 * C2DW * 365; // %body carbon /d ->  mg Carbon /(mg DW * year)

    I_T_alpha = GetFloat(iniFile, "Consumers", "I_T_alpha", -999);        //0: weight effect on assimilation


        ///.. for adults
    HS_a    = GetFloat(iniFile, "Adults", "IngestHS",      -999);
        ///.. for juveniles
    HS_j    =    GetFloat(iniFile, "Juveniles", "IngestHS",      HS_a);
        ///.. for larvae
    HS_l        = GetFloat(iniFile, "Larvae", "IngestHS", HS_a);                   // if -1 use adults' paramters
        ///Convert for Holling type 3 functional responce
    Holling_type =  GetInt(iniFile, "Consumers", "Holling_type", -999);
    if (Holling_type == 3) { //Use Holing type 3 functional response
        HS_l    = HS_l * HS_l;
        HS_a    = HS_a * HS_a;
        HS_j    = HS_j * HS_j;
    }

    AffectChl_a    = GetFloat(iniFile, "Adults", "AffectChl",      -999);
        ///.. for juveniles
    AffectChl_j    =    GetFloat(iniFile, "Juveniles", "AffectChl",  AffectChl_a);
        ///.. for larvae
    AffectChl_l        = GetFloat(iniFile, "Larvae", "AffectChl", AffectChl_a);                   // if -1 use adults' paramters



    cout << "Maintenance " << endl;
    T_a = GetFloat(iniFile, "Adults", "T", -999);  //maintenance coefficient
    T_j = GetFloat(iniFile, "Juveniles", "T", T_a);  //maintenance coefficient //convert [1/day] -> [1/year]
    T_l = GetFloat(iniFile, "Larvae", "T", T_a);  //maintenance coefficient //convert [1/day] -> [1/year]

    T_a = 365*T_a;//convert [1/day] -> [1/year]
    T_j = 365*T_j;
    T_l = 365*T_l;



    Const_growth=GetFloat(iniFile, "Debug", "Const_growth", -999); // [year^-1] if > 0 then krill increases in size with this growth rate. To test the model.


/// Metabolic rate of adults
    Metabolic_rate_min      = GetFloat(iniFile, "Adults", "Metabolic_rate_min", -999);      //the lowest level of the metabolic rate for adults
    Metabolic_bestday       = GetFloat(iniFile, "Adults", "Metabolic_bestday", -999);        //the day of year when the metabolic rate is highest
    Metabolic_high_duration = GetFloat(iniFile, "Adults", "Metabolic_high_duration", -999); // Duration of high metabolic period

    Metabolic_bestday       = Metabolic_bestday/365;      // [day] -> [year]
    Metabolic_high_duration = Metabolic_high_duration /365;      // [day] -> [year]

    pi_Metab_duration_2     = (M_PI * Metabolic_high_duration/2) * (M_PI * Metabolic_high_duration/2);


///Resource dynamics
    cout << "Resource dynamics " << endl;
    Water_col_ice_layer_ratio= GetFloat(iniFile, "Resource", "Water_col_ice_layer_ratio", -999); //=20;   the ratio between the depth of the water column and of the ice algae layer

    //Use annular oscillation mode, ENSO or something else to generate Chl_K
    Use_Index       = GetInt(iniFile,    "Resource", "Use_Index", -999);   // ;  0 - no index, 1 SAM (monthly 1979-2015), K_Ph = K_0 * exp(SAM * Ind_regression)
    Ind_regression  = GetFloat(iniFile, "Resource", "Ind_regression",   -999);;  //=0.5   ;  the slope in the dependence above
    Index_starts    = GetInt(iniFile,    "Resource", "Index_starts", -999);    //=1979    ;  year when the index starts, this will be year Zero in the model results
    Index_ends      = GetInt(iniFile,    "Resource", "Index_ends", -999);      //=2015      ;  the last full year of the index.

    ///Water column chlorophyll
    Perturb_year    = GetInt(iniFile,    "Resource", "Perturb_year", -999); //;   [years]       if e.g. Perturb_year = 3 then each third year take perturbed paramters
    C2Chl           = GetFloat(iniFile, "Resource", "C2Chl",        -999);           // [mg C/mg Chl-a] Carbon to Chlorophill ratio
    Chl_K           = GetFloat(iniFile, "Resource", "Chl_K",        -999);       // Convert into carbon.     Carrying capasity for Chl production
    Chl_K_pert      = GetFloat(iniFile, "Resource", "Chl_K_pert",    Chl_K);       // Convert into carbon.     Carrying capasity for Chl production
    Chl_infl        = GetFloat(iniFile, "Resource", "Chl_infl",     0);
    Chl_min         = GetFloat(iniFile, "Resource", "Chl_min",      -999);      //  Minimum (winter) level of Chl.
    Chl_infl_pert    = GetFloat(iniFile, "Resource", "Chl_infl_pert",  Chl_infl);  //  Minimum (winter) level of Chl.
    Chl_g_max       = GetFloat(iniFile, "Resource", "Chl_g_max",    -999);      //  Maximal growht rate of Chl
    Chl_bestday     = GetFloat(iniFile, "Resource", "Chl_bestday",  -999); //         The day of year, when the maximal production of chlrorofill is reached
    Chl_duration    = GetFloat(iniFile, "Resource", "Chl_duration", -999);//           Duration of summer, when chlorophyll net production is positive
    Chl_loss = GetFloat(iniFile, "Resource", "Chl_loss",            -999);         //       Natural loss rate of Chl-a
    ///Ice algae
    Chl_ice_K       = GetFloat(iniFile, "Resource", "Chl_ice_K",       -999);//  Carrying capasity for ice algae
    Chl_ice_K_pert  = GetFloat(iniFile, "Resource", "Chl_ice_K_pert",  Chl_ice_K);//  Carrying capasity for ice algae
    Chl_ice_min     = GetFloat(iniFile, "Resource", "Chl_ice_min",     Chl_min); //   Minimum (summer) level of ice algae, if -1 then take Chl_min
    Chl_ice_g_max   = GetFloat(iniFile, "Resource", "Chl_ice_g_max",   Chl_g_max);//       Maximal growht rate of Ice Algae if -1 then take Chl_g_max
    Chl_ice_bestday = GetFloat(iniFile, "Resource", "Chl_ice_bestday", -999);//     The day of year, when the maximal production of ice algae is reached
    Chl_ice_duration= GetFloat(iniFile, "Resource", "Chl_ice_duration",-999);//       Duration of ice algae period
    Chl_ice_loss    = GetFloat(iniFile, "Resource", "Chl_ice_loss",    Chl_loss);
    C2Chl_ice       = GetFloat(iniFile, "Resource", "C2Chl_ice",       C2Chl);// Carbon to ice-algae-chlorophill ratio

    ///Convert units
    Chl_K           = C2Chl * Chl_K;        // [mg Chl] -> [mg C] Convert into carbon.
    Chl_K_pert      = C2Chl * Chl_K_pert;   // [mg Chl] -> [mg C] Convert into carbon.
    Chl_infl        = C2Chl * Chl_infl;      // [mg Chl] -> [mg C]
    Chl_infl_pert   = C2Chl * Chl_infl_pert; // [mg Chl] -> [mg C]
    Chl_infl        = 365 * Chl_infl;       // [1/day] -> [1/year]
    Chl_infl_pert   = 365 * Chl_infl_pert; // [mg Chl] -> [mg C]
    Chl_min         = C2Chl * Chl_min;      // [mg Chl] -> [mg C]
    Chl_g_max       = 365 * Chl_g_max;      // [1/day] -> [1/year]
    Chl_bestday     = Chl_bestday/365;      // [day] -> [year]
    Chl_duration    = Chl_duration/365;     // [day] -> [year]
    Chl_loss        = 365 * Chl_loss;       // [1/day] -> [1/year]

    Chl_ice_K           = C2Chl_ice * Chl_ice_K;        // [mg Chl] -> [mg C] Convert into carbon.
    Chl_ice_K_pert      = C2Chl_ice * Chl_ice_K_pert;   // [mg Chl] -> [mg C] Convert into carbon.
    Chl_ice_min         = C2Chl_ice * Chl_ice_min;      // [mg Chl] -> [mg C]
    Chl_ice_g_max       = 365 * Chl_ice_g_max;      // [1/day] -> [1/year]
    Chl_ice_bestday     = Chl_ice_bestday/365;      // [day] -> [year]
    Chl_ice_duration    = Chl_ice_duration/365;     // [day] -> [year]
    Chl_ice_loss        = 365 * Chl_ice_loss;       // [1/day] -> [1/year]

    pi_Chl_duration_2       = (M_PI * Chl_duration/2) * (M_PI * Chl_duration/2);     // constant (pi * Chl_duration/2)^2
    pi_Chl_ice_duration_2   = (M_PI * Chl_ice_duration/2) * (M_PI * Chl_ice_duration/2);  // constant (pi * Chl_ice_duration/2)^2



///splitting of assimilated carbon between growth and reproduction K(L)
    cout << "Growth - reproduction " << endl;
    L_repr_mid = GetFloat(iniFile, "Consumers", "L_repr_mid", -999);
    L_delta = GetFloat(iniFile, "Consumers", "L_delta", -999);
    L_repr_min = GetFloat(iniFile, "Consumers", "L_repr_min", -999);


    Esp_ovary = GetFloat(iniFile, "Consumers", "Esp_ovary", -999); ;  // the ratio eggs dry weight to ovary dry weigth


///Emprions paramters
    cout << "Embryones " << endl;
    Egg_hatch = GetFloat(iniFile, "Embryones", "Egg_hatch", -999); // the number of eggs which survive
///Egg size and weight
    EggWeigth = GetFloat(iniFile, "Embryones", "EggWeigth", -999);
    EggSize = GetFloat(iniFile, "Embryones", "EggSize", -999);


///Mortality
    cout << "Mortality " << endl;
    Mortality_e         = GetFloat(iniFile, "Embryones",    "Mortality", -999); //the base mortality of embryones;
    Mortality_l         = GetFloat(iniFile, "Larvae",       "Mortality", -999); //mortality of larvae
    Mortality_starv_l   = GetFloat(iniFile, "Larvae",       "Mortality_starv", -999); //starvation mortality of larvae
    Mortality_j         = GetFloat(iniFile, "Juveniles",    "Mortality", -999); //the mortality of juvenails
    Mortality_a         = GetFloat(iniFile, "Adults",       "Mortality", -999); //the mortality of adults
    Mort_factor         = GetFloat(iniFile, "Debug",        "Mort_factor", -999);  //if zero then indifiduals do not die independend of the mortality level


    Use_allometric_mortality = GetFloat(iniFile, "Consumers", "Use_allometric_mortality", -999); //  =1 if we use allometrix mortality accoring McGurk 1986
    Mortality_allom_w        = GetFloat(iniFile, "Consumers", "Mortality_allom_w",        -999); //  =1.92          ; [1/year]  1.92 = 5.26 * 0.001 * 365
    Mortality_allom_sigma    = GetFloat(iniFile, "Consumers", "Mortality_allom_sigma",    -999); //  =-0.25
    Mortality_allom_max      = GetFloat(iniFile, "Consumers", "Mortality_allom_max",      -999); //  =10          ; [1/year]; 10 the maximal allowed mortality


///Distributions
    cout << "Distributions " << endl;
    Distrib_length = GetFloat(iniFile, "Model",    "Distrib_length", -999);  //=1    ;   1 to save length distribution
    Distrib_len_min = GetFloat(iniFile, "Model",    "Distrib_len_min", -999);     //=0    ;   minimal length
    Distrib_len_max = GetFloat(iniFile, "Model",    "Distrib_len_max", -999);     //=70   ;  [mm] maximal length
    Distrib_len_bins = GetFloat(iniFile, "Model",   "Distrib_len_bins", -999);    //=50;  number of bins
    if (Distrib_length){
        ofstream file_len("./krill_len.txt");
		file_len.close();
    }

    Distrib_age = GetFloat(iniFile, "Model",    "Distrib_age", -999);  //=1    ;   1 to save age distribution
    Distrib_age_min = GetFloat(iniFile, "Model",    "Distrib_age_min", -999);     //=0    ;   minimal age
    Distrib_age_max = GetFloat(iniFile, "Model",    "Distrib_age_max", -999);     //=7   ;  [years] maximal age
    Distrib_age_bins = GetFloat(iniFile, "Model",   "Distrib_age_bins", -999);    //=50;  number of bins
    if (Distrib_age){
        ofstream file_len("./krill_age.txt");
		file_len.close();
    }

    Distrib_fec = GetFloat(iniFile, "Model",    "Distrib_fec", -999);     //=1       ;   1 to save fecandity distrib over length
    if (Distrib_fec ){
        ofstream file_fec;
		file_fec.open( "./krill_fec.txt");
		file_fec.close();
    }

    Distrib_bottleneck = GetFloat(iniFile, "Model",    "Distrib_bottleneck", -999);    //=1       ;   1 to save (Sb beta(s, R) + g(s, R))/s - d(s, R)
    if (Distrib_bottleneck ){
        ofstream file_bn;
		file_bn.open( "./krill_bottleneck.txt");
		file_bn.close();
    }

    Distrib_output_intv = GetFloat(iniFile, "Model",     "Distrib_output_intv", -999) ;  //=0.1  ;   [year]          output interval of distributions

    double Length_init = GetFloat(iniFile, "Debug",     "Length_init", -999) ;  //[mm] the initial length of all cohorts ;
    if (Length_init )
    {
        double Weigth_init = L2W(Length_init);
        for (int i=0; i < cohort_no[CONS]; i++) {
            pop[CONS][i][weight]= Weigth_init;
        }

    }

    int Random_Init = GetInt(iniFile, "Model",     "Random_Init", -999) ;



    fflush(stderr);
    UpdateIDcards(env, pop, 0);
    return;

}




/*
 *===========================================================================
 *
 *	SPECIFICATION OF THE NUMBER AND VALUES OF BOUNDARY POINTS
 *
 *===========================================================================
 */

void	SetBpointNo(double *env, population *pop, int *bpoint_no)

{
    bpoint_no[CONS] = 1;

    return;
}




/*===========================================================================*/

void	SetBpoints(double *env, population *pop, population *bpoints)
{

    bpoints[0][0][age]=0.0;
    bpoints[0][0][weight]=EggWeigth;

    return;

}





/*
 *===========================================================================
 *
 *			SPECIFICATION OF DERIVATIVES
 *
 *===========================================================================
 */

void	Gradient(double *env,	  population *pop,     population *ofs,
                 double *envgrad, population *popgrad, population *ofsgrad,
                 population *bpoints)

{
    register int		i;
    double		Mortality = 0.0;
    double      TotalFecundity = 0.0;
    double		tot_grazing_chl = 0.0;
    double      tot_grazing_ice = 0.0;

    UpdateIDcards(env, pop, 0);

    for (i=0; i<cohort_no[CONS]; i++) {

        TotalFecundity = TotalFecundity + 0.5 * pop[CONS][i][number] * Egg_hatch * popIDcard[CONS][i][IDfecundity];
        Mortality = Mort_factor * popIDcard[CONS][i][IDmort];
        popgrad[CONS][i][number] = -Mortality * pop[CONS][i][number];
        popgrad[CONS][i][age]    =  1.0;
        if (Const_growth) // if Const_growth > 1 then krill will increase its size exponentially, just to test the model
            popgrad[CONS][i][weight] = Const_growth * pop[CONS][i][weight];
        else
            popgrad[CONS][i][weight] = popIDcard[CONS][i][IDgrowth];

        tot_grazing_chl += popIDcard[CONS][i][IDingest_chl] * pop[CONS][i][number] * popIDcard[CONS][i][IDAffectChl];
        tot_grazing_ice += popIDcard[CONS][i][IDingest_ice] * pop[CONS][i][number] * Water_col_ice_layer_ratio;

    }

    envgrad[0] = 1.0;                                                       //time



   //Growth rate of water column chlorophyll and ice algae
    double chl_growth(0);
    double t1 = time - Chl_bestday;
    if (fabs(t1 - round(t1)) < Chl_duration/2)
        chl_growth  = Chl_g_max;
    else
        chl_growth  = 0;
    double ice_alg_gr = Chl_ice_g_max * exp((cos(2 * M_PI * (time - Chl_ice_bestday))  - 1)/pi_Chl_ice_duration_2);

    double  Chl_infl_cur(Chl_infl), Chl_ice_K_cur(Chl_ice_K); //Chl_K_cur(Chl_K),
    Chl_K_cur = Chl_K;


    if (Use_Index)  //Use an index of external perturbations
    {
        double Index_Year = Modiv(time, Index_ends - Index_starts + 1); //year
        int iIndex_Year   = int(Index_Year); //index of this year in SAM array
        int iIndex_Month   = int((Index_Year - iIndex_Year)*12); //index of this month in SAM array
        double SAM = SAM_1979_2015[iIndex_Year][iIndex_Month];
        Chl_K_cur = Chl_K * exp(SAM * Ind_regression);  //Current carrying capacity
    }
    else if (Perturb_year != 0)
    {
        unsigned int year = round(time - Chl_bestday); //summer year
        //srand(year);
        init_gen_rand(year + Chl_K_pert); //use the same rand for the year
        if (Perturb_year > 0)
        {
            if ( genrand_real1() * Perturb_year< 1)  //perturbation with probability 1/Perturb_year;
            {
                  Chl_K_cur      = Chl_K_pert;
                  Chl_infl_cur   = Chl_infl_pert;
            }
        }
        else if (Perturb_year == -99)  // assign random values in the given interval
        {
           Chl_K_cur      = genrand_real1() * (Chl_K_pert - Chl_K) + Chl_K;
           Chl_infl_cur   = Chl_infl_pert;

        }
        else if (Perturb_year < 0) //periodic oscillations with given period
        {
           if (  IntPart(Modiv(year, -Perturb_year)) == 0)
            {
               Chl_K_cur = Chl_K_pert;
               Chl_infl_cur   = Chl_infl_pert;
            }
            else
            {
              Chl_K_cur = Chl_K_cur;
            }
        }

        year = round(time - Chl_ice_bestday);  //winter year
        init_gen_rand(year); //use the same rand for the year
        //srand();
        if ( genrand_real1() * Perturb_year< 1)  //perturbation with probability 1/Perturb_year;
        {
             Chl_ice_K_cur = Chl_ice_K_pert;
        }
    }
    envgrad[1] = chl_growth * chl     * (1 -     chl/Chl_K_cur    ) - Chl_loss     * (chl     - Chl_min)     + Chl_infl_cur - tot_grazing_chl;                        //chlorophyll
    envgrad[2] = ice_alg_gr * ice_alg * (1 - ice_alg/Chl_ice_K_cur) - Chl_ice_loss * (ice_alg - Chl_ice_min)                - tot_grazing_ice;                        //ice algae


    ofsgrad[0][0][number] = -Mortality_e * ofs[0][0][number]
                                + TotalFecundity;
    ofsgrad[0][0][age]    = -Mortality_e * ofs[0][0][age]
                            + ofs[0][0][number];
    ofsgrad[0][0][weight] = -Mortality_e * ofs[0][0][weight];  //eggs weight and mortality are constant

    return;

}




/*
 *===========================================================================
 *
 *		SPECIFICATION OF BETWEEN COHORT CYCLE DYNAMICS
 *
 *===========================================================================
 */

void	InstantDynamics(double *env, population *pop, population *ofs)

{

    /*
     * This function can also be defined as:

     void	CycleEnd(double *env, population *pop, population *ofs)

     * The function names SetBpointNo and CycleStart are synonyms.
     *
     * Apart from this, the function header above should not be changed.
     *
     * This routine is called at the end of each cohort cycle. The routine
     * can be used to implement any type of instantaneous dynamics, occurring
     * between two subsequent cohort cycles. It can, for instance, be used
     * for a pulsed, instantaneous reproduction process or to set the number
     * of individuals in cohorts that have reached their maximum lifespan to
     * 0.
     * Note that the transformation of boundary cohorts into internal cohorts
     * has already been performed and that the boundary cohorts are hence
     * characterized by the number of individuals and their transformed
     * moments (usually denoted by the symbol mu).
     * In an instantaneous reproduction process the i_state of the offspring
     * can therefore be simply specified in terms of the mean i_state!
     *
     *     NB : -	The integer array "cohort_no[]" is globally available and
     *		denotes the number of internal cohorts present in each
     *		structured population.
     *	      - The integer array "bpoint_no[]" is globally available and
     *		denotes the number of boundary cohorts present in each
     *		structured population during the current cohort cycle.
     *	      - The offspring number can be zero. If used in a division, check
     *		for this equality to zero!!
     */

    register int		i;

    for(i=0; i<cohort_no[0]; i++) {		/* Empty cohorts that have  */
        /* reached maximum age      */
        if(pop[0][i][age] > Age_max) pop[0][i][number] = 0;
    }

    return;


   UpdateIDcards(env, pop, 0);

    return;
}




/*
 *===========================================================================
 *
 *			SPECIFICATION OF OUTPUT VARIABLES
 *
 *===========================================================================
 */

void	DefineOutput(double *env, population *pop, double *output)

{
    register int		i;

    UpdateIDcards(env, pop, 0);

#if (BIFURCATION == 1)
    LabelState(CONS, "Bif. parameter = %.4f   T = %.0f", parameter[BifParIndex], env[0]);
#endif /* BIFURCATION */


    double Abund(0.0), Biomass(0.0), MeanAge(0.0);
    double Biomass_e(0.0), Biomass_l(0.0), Biomass_j(0.0), Biomass_a (0.0);
    double Abund_e(0.0), Abund_l(0.0), Abund_j(0.0), Abund_a(0.0);
    double MeanLength(0), MeanLength_e(0), MeanLength_l(0), MeanLength_j(0), MeanLength_a(0);
    double Mean_grrate(0), Mean_grrate_e(0), Mean_grrate_l(0), Mean_grrate_j(0), Mean_grrate_a(0);
    double Tot_ingest_l_chl(0), Tot_ingest_l_ice(0), Tot_ingest_j(0), Tot_ingest_a(0);
    double Mean_stage(0);
    double Mean_mort(0), Mean_mort_e(0), Mean_mort_l(0), Mean_mort_j(0), Mean_mort_a(0);
    double MeanFecundity(0);
    double Length(0), Weight(0), Number(0), Age(0);
    double Mean_metab_rate_a(0);
    int iState(0);

    THist1D  Hist_len(Distrib_len_min, Distrib_len_max, Distrib_len_bins);
    THist1D  Hist_age(Distrib_age_min, Distrib_age_max, Distrib_age_bins);
    THist1D  Hist_fec(Distrib_len_min, Distrib_len_max, Distrib_len_bins);
    THist1D  Hist_bn(Distrib_len_min, Distrib_len_max, Distrib_len_bins);


    for (i=0; i<cohort_no[CONS]; i++) {
        Weight = pop[CONS][i][weight];
        Number = pop[CONS][i][number];
        Age    = pop[CONS][i][age];
        Length = W2L(Weight);
        iState = popIDcard[CONS][i][IDstate];
        Abund   += Number;
        Biomass += Number * Weight;
        Mean_grrate += Number * popIDcard[CONS][i][IDgrowth];
        Mean_mort   += Number * popIDcard[CONS][i][IDmort];

        MeanAge     += Number * pop[CONS][i][age];
        MeanLength  += Number * Length;

        Mean_stage  +=  Number * popIDcard[CONS][i][IDstate];

        MeanFecundity += Number * popIDcard[CONS][i][IDfecundity];

        if ( (Distrib_length) && (time - Distrib_outout_time)>-1e-7){
            Hist_len.Add(Length, Number);
        };

        if ( (Distrib_age)    && (time - Distrib_outout_time)>-1e-7){
            Hist_age.Add(Age, Number);
        };

        if (  (Distrib_fec )  && (time - Distrib_outout_time)>-1e-7){
            Hist_fec.Add(Length, popIDcard[CONS][i][IDfecundity]);
        };

        if (  (Distrib_bottleneck )   && (time - Distrib_outout_time)>-1e-7){
            Hist_bn.Add(Length,  pop[CONS][i][number] * ( (EggWeigth * popIDcard[CONS][i][IDfecundity] +  popIDcard[CONS][i][IDgrowth])/Weight - popIDcard[CONS][i][IDmort]));
        };



        switch (iState) {
        case 1: { //Embryos
                Abund_e         += Number;
                Biomass_e       += Number * Weight;
                MeanLength_e    += Number * Length;
                Mean_grrate_e   += Number * popIDcard[CONS][i][IDgrowth];
                Mean_mort_e     += Number * popIDcard[CONS][i][IDmort];

                break;
            }
        case 2: { // larvae consume chl and ice algae
                Abund_l         += Number;
                Biomass_l       += Number * Weight;
                MeanLength_l    += Number * Length;
                Mean_grrate_l   += Number * popIDcard[CONS][i][IDgrowth];
                Mean_mort_l     += Number * popIDcard[CONS][i][IDmort];
                Tot_ingest_l_chl  += Number * popIDcard[CONS][i][IDingest_chl];
                Tot_ingest_l_ice  += Number * popIDcard[CONS][i][IDingest_ice] * Water_col_ice_layer_ratio;
                break;
            }
        case 3:   //juveniles, consume only chlorophyll
                Abund_j         += Number;
                Biomass_j       += Number * Weight;
                MeanLength_j    += Number * Length;
                Mean_grrate_j   += Number * popIDcard[CONS][i][IDgrowth];
                Mean_mort_j     += Number * popIDcard[CONS][i][IDmort];
                Tot_ingest_j      += Number * popIDcard[CONS][i][IDingest_chl];
                break;
        case 4: { //adults , consume only chlorophyll
                Abund_a         += Number;
                Biomass_a       += Number * Weight;
                MeanLength_a    += Number * Length;
                Mean_grrate_a   += Number * popIDcard[CONS][i][IDgrowth];
                Mean_mort_a     += Number * popIDcard[CONS][i][IDmort];
                Mean_metab_rate_a    += Number * popIDcard[CONS][i][IDmetabrate];
                Tot_ingest_a    += Number * popIDcard[CONS][i][IDingest_chl];
                break;
            }
        }
    }
    MeanAge         = SaveDiv(MeanAge       ,Abund);
    MeanLength      = SaveDiv(MeanLength    ,Abund);
    Mean_stage      = SaveDiv(Mean_stage    ,Abund);
    MeanFecundity   = SaveDiv(MeanFecundity ,Abund);
    Mean_grrate     = SaveDiv(Mean_grrate   ,Abund);
    Mean_mort       = SaveDiv(Mean_mort     ,Abund);

    MeanLength_e  = SaveDiv(MeanLength_e,Abund_e);
    MeanLength_l  = SaveDiv(MeanLength_l,Abund_l);
    MeanLength_j  = SaveDiv(MeanLength_j,Abund_j);
    MeanLength_a  = SaveDiv(MeanLength_a,Abund_a);

    Mean_grrate_e     = SaveDiv(Mean_grrate_e,Abund_e);
    Mean_mort_e       = SaveDiv(Mean_mort_e  ,Abund_e);
    Mean_grrate_l     = SaveDiv(Mean_grrate_l,Abund_l);
    Mean_mort_l       = SaveDiv(Mean_mort_l  ,Abund_l);
    Mean_grrate_j     = SaveDiv(Mean_grrate_j,Abund_j);
    Mean_mort_j       = SaveDiv(Mean_mort_j  ,Abund_j);
    Mean_grrate_a     = SaveDiv(Mean_grrate_a,Abund_a);
    Mean_mort_a       = SaveDiv(Mean_mort_a  ,Abund_a);
    Mean_metab_rate_a = SaveDiv(Mean_metab_rate_a,Abund_a);


    output[ 0] = cohort_no[0];
    output[ 1] = chl;
    output[ 2] = ice_alg;

    output[ 3] =       Biomass;
    output[ 4] =         Abund;
    output[ 5] =       MeanAge;
    output[ 6] =    MeanLength;
    output[ 7] = MeanFecundity;
    output[ 8] =   Mean_grrate;
    output[ 9] =     Mean_mort;

    output[10] =     Biomass_e;
    output[11] =       Abund_e;
    output[12] =  MeanLength_e;
    output[13] = Mean_grrate_e;
    output[14] =   Mean_mort_e;

    output[15] =     Biomass_l;
    output[16] =       Abund_l;
    output[17] =  MeanLength_l;
    output[18] = Mean_grrate_l;
    output[19] =   Mean_mort_l;

    output[20] =    Biomass_j;
    output[21] =    Abund_j;
    output[22] =    MeanLength_j;
    output[23] =    Mean_grrate_j;
    output[24] =    Mean_mort_j;

    output[25] =    Biomass_a;
    output[26] =    Abund_a;
    output[27] =    MeanLength_a;
    output[28] =    Mean_grrate_a;
    output[29] =    Mean_mort_a;
    output[30] =    Mean_metab_rate_a;
    output[31] =    Mean_stage;

    output[32] =    Chl_K_cur;


    output[33] =    Tot_ingest_l_chl;
    output[34] =    Tot_ingest_l_ice;
    output[35] =    Tot_ingest_j;
    output[36] =    Tot_ingest_a;


    if ((time - Distrib_outout_time)> -1e-7){
    //output distributions
        Distrib_outout_time = time + Distrib_output_intv;
        if (Distrib_length){
            ofstream file_len( "krill_len.txt", ios::out|ios::app);
            file_len << time;
            for (i = 0; i< Hist_len.XSize; i++){
                file_len  << '\t' << Hist_len.values[i].Sum;
            }
            file_len << endl;
            file_len.close();
        }

        if (Distrib_age){
            ofstream file_age( "krill_age.txt", ios::out|ios::app);
            file_age << time;
            for (i = 0; i< Hist_age.XSize; i++){
                file_age  << '\t' << Hist_age.values[i].Sum;
            }
            file_age << endl;
            file_age.close();
        }


        if (Distrib_fec ){
            ofstream file_fec( "krill_fec.txt", ios::out|ios::app);
            file_fec << time;
            for (i = 0; i< Hist_fec.XSize; i++){
                file_fec  << '\t' << Hist_fec.values[i].Mean();
            }
            file_fec << endl;
            file_fec.close();
        }

        if (Distrib_bottleneck ){
            ofstream file_fec( "krill_bottleneck.txt", ios::out|ios::app);
            file_fec << time;
            for (i = 0; i< Hist_bn.XSize; i++){
                file_fec  << '\t' << Hist_bn.values[i].Mean();
            }
            file_fec << endl;
            file_fec.close();
        }
    }


    return;
}





inline double fermi(double x, double alpha) {           // Fermi function == 1 for negative x
    return 1./(1.+exp(x/alpha));                        //and 0 for positive x

}


inline double fch(double FromValue, double ToValue, double z, double AtValue, double Alpha) {
    return  (FromValue - ToValue)/(1.+exp((z-AtValue)/Alpha)) +  ToValue;   // A smooth (Fermi) change from
}                                                                           // FromValue to ToValue at the AtValue

inline double W2L(double Weight) {
    if (Weight > EggWeigth)
        return  exp(0.5 * (-Sigma1 + sqrt(Sigma1 * Sigma1  + 4 * Sigma2 * log(Weight/Cw)))/Sigma2);
    else
        return  EggSize;
};

inline double L2W(double Length) {
    if (Length > EggSize)
        return  Cw * pow(Length, Sigma1 + Sigma2 * log(Length));
    else
        return EggWeigth;
};


inline double Split_K(double Length)
{
	//Get the part of the assimilated food invested into growth (the rest is egg production)
	//return 1./(1.+exp((Weight-ReproductiveWeight)/60));
    return 1./(1.+exp((Length-L_repr_mid)/L_delta));
};

/*==============================================================================*/

static void	UpdateIDcards(double *env, population *pop, int setfec)

{
    register int		i;
    double MetabRate(0);
    double Weight(0);
    double Length(0);
    double Age(0);
    double I_max(0);

    double ice_alg2 = ice_alg;// *  ice_alg;
    double time_val = time;
    double chla2    = chl;//    *  chl;

    double ice_alg_larv = ice_alg * ice_alg/(chl + ice_alg);  //proportions between ice_algae and chlorophyll densities
    double chl_larv     = chl    * chl   /(chl + ice_alg);  //   for active switching of krill larvae

    double Split_K_val(0);
    int iState(0);
    double Delta(0);
    double Mortality_starv(0);
    double WeightEff(1);
    //double T;




    for (i=0; i<cohort_no[CONS]; i++) {

        Weight      = pop[CONS][i][weight];
        if (I_T_alpha != 0)
        {
            WeightEff = pow(Weight, I_T_alpha);
        }
        Length      = W2L(Weight);
        Split_K_val = Split_K(Length);
        Age         = pop[CONS][i][age];
/// define the state of each cohort.   State 1= emrions 2 = larvae, 3 Juveniles, 4-Adults
        if (Mort_factor==0) //debugging regime, the stage is length dependent
        {
            if (Age < Age_l) {
                popIDcard[CONS][i][IDstate] = 1; // Embryos do not consume anything
            } else if (Length < L_larv_juv) {
                popIDcard[CONS][i][IDstate] = 2; // consume Chl-a and ice algae

            } else if (Length < L_repr_min) {
                popIDcard[CONS][i][IDstate] = 3; // Consume Chl-a, can decrease metabolic activity
            } else {
                popIDcard[CONS][i][IDstate] = 4; // Consume Chl-a, can decrease metabolic activity and can reproduce if large enough
            }
        }
        else {
            if (Age < Age_l) {
                popIDcard[CONS][i][IDstate] = 1; // Embryos do not consume anything
            } else if (Age < Age_j) {
                popIDcard[CONS][i][IDstate] = 2; // consume Chl-a and ice algae
            } else if (Age < Age_a) { //was commented because otherwise there is a jump in growth rate at the transition form  j to a
                popIDcard[CONS][i][IDstate] = 3; // Consume Chl-a, can decrease metabolic activity
            } else {
                popIDcard[CONS][i][IDstate] = 4; // Consume Chl-a, can decrease metabolic activity and can reproduce if large enough
            }
        }
       iState = popIDcard[CONS][i][IDstate];

///define MetabolicRate
        if (iState >= 3) {//juveniles and adults can change the metabolic rate
            MetabRate =  exp(   (cos(2 * M_PI * (time - Metabolic_bestday))  - 1)/pi_Metab_duration_2   );
            //fch(0, 1,  Modiv(time, 1.0), TimeLowMetabSpring, YearAlpha) * fch(1, 0,  Modiv(time, 1.0), TimeLowMetabAutumn, YearAlpha);
            MetabRate =  Metabolic_rate_min + (1 - Metabolic_rate_min) * MetabRate;
        } else {
            MetabRate = 1;
        }
        popIDcard[CONS][i][IDmetabrate] = MetabRate;



        //I_max = fch(I_max_0, I_max_60, Length, L_repr_min, L_delta);
        I_max = fch(I_max_0, I_max_60, Length, L_larv_juv, L_delta_l_j) * WeightEff;
        //I_max = I_max_0 + (I_max_60 - I_max_0)/60. * Length;

/// Ingestion of ice algae per individual and maintenance
        switch (iState) {
        case 1: { //Embryos
                popIDcard[CONS][i][IDingest_chl] = 0;
                popIDcard[CONS][i][IDingest_ice] = 0;
                popIDcard[CONS][i][IDmaint] = 0; // Embryos have no maintenance
                popIDcard[CONS][i][IDAffectChl] = 0;
                break;
            }
        case 2: { // larvae ingest chl and ice algae
                popIDcard[CONS][i][IDingest_ice] = MetabRate * I_max    * ice_alg_larv /(chl_larv + ice_alg_larv + HS_l) *  Weight;
                popIDcard[CONS][i][IDingest_chl] = MetabRate * I_max    * chl_larv     /(chl_larv + ice_alg_larv + HS_l) *  Weight;
                popIDcard[CONS][i][IDmaint] = T_l * Weight;
                popIDcard[CONS][i][IDAffectChl] = AffectChl_l;
                break;
            }
        case 3: {  //juveniles, consume only chlorophyll
                popIDcard[CONS][i][IDingest_chl] = MetabRate * I_max *   chla2 /(chla2 + HS_j) *  Weight;
                popIDcard[CONS][i][IDingest_ice] = 0;
                popIDcard[CONS][i][IDmaint] = T_j * Weight;
                popIDcard[CONS][i][IDAffectChl] = AffectChl_j;
                break;
            }
        case 4: { //adults, consume only chlorophyll
                popIDcard[CONS][i][IDingest_chl] = MetabRate * I_max *   chla2 /(chla2 + HS_a) *  Weight;
                popIDcard[CONS][i][IDingest_ice] = 0;
                popIDcard[CONS][i][IDmaint] = T_a * Weight;
                popIDcard[CONS][i][IDAffectChl] = AffectChl_a;
                break;
            }
        }

        //smooth change of the maintenance rate (added 08.07.2015)
        if (iState > 1)
        {
            popIDcard[CONS][i][IDmaint] = fch(T_l, T_a , Length, L_larv_juv, L_delta_l_j) * Weight * WeightEff;
        }


///Assimilation
        popIDcard[CONS][i][IDassim] = Eps_assim * (popIDcard[CONS][i][IDingest_ice] +  popIDcard[CONS][i][IDingest_chl]);

///Net assimilation of the biomass
        popIDcard[CONS][i][IDnet_assim] = popIDcard[CONS][i][IDassim] - popIDcard[CONS][i][IDmaint];

///Fecundity
        if ( (iState == 4) &&                                              //reproduction is possible for adults (> 2 y.o.)
              (popIDcard[CONS][i][IDnet_assim] > 0) &&                    //which get enough food
              (Length > L_repr_min) )                                     //of the reproductive length
                popIDcard[CONS][i][IDfecundity] = Esp_ovary * (1 - Split_K_val ) * popIDcard[CONS][i][IDnet_assim]/EggWeigth;
        else
              popIDcard[CONS][i][IDfecundity] = 0;

///Growth
        //if positive we assume that a part of the assim food will be invested into development
         if ( popIDcard[CONS][i][IDnet_assim] > 0)                                  //which get enough food
                popIDcard[CONS][i][IDgrowth]    =                  Split_K_val   * popIDcard[CONS][i][IDnet_assim];
         else
         {
            if (Length > 1) //Only longer than 1 mm long krill can shrink
                popIDcard[CONS][i][IDgrowth]    =  popIDcard[CONS][i][IDnet_assim];
            else
                popIDcard[CONS][i][IDgrowth]    =  0;
         }


/// Mortality
        Mortality_starv =  0;
        if (iState > 1) { //find starvation mortality
            Delta =  popIDcard[CONS][i][IDassim]/popIDcard[CONS][i][IDmaint] - 1;
            if (Delta < 0 && iState> 1)  //Starvation mortality is size dependent for l-j-a
               Mortality_starv =  - fch(Mortality_starv_l, 0, Length, L_larv_juv, L_delta_l_j) * Delta ;
        }

        if (Use_allometric_mortality)     {
            popIDcard[CONS][i][IDmort] = Mortality_allom_w * pow(Weight, Mortality_allom_sigma);
            if (popIDcard[CONS][i][IDmort]> Mortality_l) popIDcard[CONS][i][IDmort] = Mortality_allom_max; //should not be greater than maximal allometric mort
            popIDcard[CONS][i][IDmort] = popIDcard[CONS][i][IDmort] + Mortality_starv;
        }
        else {
            switch (iState) {
            case 1: { //Embryos
                    popIDcard[CONS][i][IDmort] = Mortality_e;
                    break;
                }
            case 2: { //larvae
                    popIDcard[CONS][i][IDmort] = Mortality_l + Mortality_starv;
                    break;
                }
            case 3: {  //juveniles
                    popIDcard[CONS][i][IDmort] = Mortality_j + Mortality_starv;
                    break;
                }
            case 4: { //adults
                    popIDcard[CONS][i][IDmort] = Mortality_a + Mortality_starv;
                    break;
                }
            }
        }

    }
    return;
}



      /*==============================================================================*/


