 /******************************************************************
 *                                                                *
 * File          : mymath.cpp                                     *
 * Written by    : Alexei B. Ryabov                               *
 * e-mail        : alryabov@gmail.com                             *
 * Version of    : 24 July 2009                                   *
 *----------------------------------------------------------------*
 * This is the implementation file for mathematical routine        *
 *                                                                *
 * You may use this file as you wish in your programs.            *
 * But you should leave this title                                *
 *****************************************************************/

#ifndef mymathc
#define mymathc

#include "mymath.h"
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <iostream>
#include <fstream>
using namespace std;

const double DblPi = 2.0 * M_PI;
const double HalfPi = M_PI/2.0;
const double Pi34 = M_PI * 3.0/4.0;


unsigned long IntRand(unsigned long a)
{
    return (unsigned long)((double(rand())/double(RAND_MAX)) * double(a));
};

double FloatRand(double a)
{
    return (a * rand())/(double)RAND_MAX;
};

double DoubleRand()
{
    unsigned long Res;
    Res = rand();
    Res <<= __Bytes;
    Res += rand();
    return (double)(Res)/__MyRandMax;
};

 double randomiz(double a)
{
    return FloatRand(a);
}


/* normally distributed gaussian noise zero mean, unit var  */
float gauss(long *idum)
{
	float ran1(long *idum);
	static int iset=0;
	static float gset;
	float fac,rsq,v1,v2;

	if  (iset == 0) {
		do {
			v1=2.0*ran1(idum)-1.0;
			v2=2.0*ran1(idum)-1.0;
			rsq=v1*v1+v2*v2;
		} while (rsq >= 1.0 || rsq == 0.0);
		fac=sqrt(-2.0*log(rsq)/rsq);
		gset=v1*fac;
		iset=1;
		return v2*fac;
	} else {
		iset=0;
		return gset;
	}
}



/*  Random number generator */

// y=ran1(&idum);

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

float ran1(long *idum)
{
	int j;
	long k;
	static long iy=0;
	static long iv[NTAB];
	float temp;

	if (*idum <= 0 || !iy) {
		if (-(*idum) < 1) *idum=1;
		else *idum = -(*idum);
		for (j=NTAB+7;j>=0;j--) {
			k=(*idum)/IQ;
			*idum=IA*(*idum-k*IQ)-IR*k;
			if (*idum < 0) *idum += IM;
			if (j < NTAB) iv[j] = *idum;
		}
		iy=iv[0];
	}
	k=(*idum)/IQ;
	*idum=IA*(*idum-k*IQ)-IR*k;
	if (*idum < 0) *idum += IM;
	j=iy/NDIV;
	iy=iv[j];
	iv[j] = *idum;
	if ((temp=AM*iy) > RNMX) return RNMX;
	else return temp;
}
#undef IA
#undef IM
#undef AM
#undef IQ
#undef IR
#undef NTAB
#undef NDIV
#undef EPS
#undef RNMX

void RandSeed()
{
    time_t t;
    srand((unsigned) time(&t));
}

double Modiv(double c, double b)
{
    return (c < 0) ? fmod(c,b) + b : fmod(c,b);
};



double pow10my_(int p)
{
    double res = 1;
    int j;
    if (p >=0 )
    {
        for(j=1;j<=p;j++)
        {
            res *=10;
        }
    }
    else
        for(j=-1;j>=p;j--)
        {
            res /=10;
        }
    return res;
};



int IntPart(double d)
{
    return int(floor(d));
}


double GetAngle(double x, double y)
{
    double Angle;
    if (x == 0)
    {
        if (y != 0)
            Angle = HalfPi;
        else
            Angle = 0;
    }
    else
        Angle = atan(y/x);
    if (x < 0)
        Angle += M_PI;
    return Angle;
}

double GetAbsVal(double x, double y)
{
    return sqrt(x * x + y * y);
}


//-----------------------------------------THist---------------------------------------------------------
THist::THist(double XMin_, double XMax_, double YMin_, double YMax_, int XSize_, int YSize_)
{
    XMin = XMin_;
    XMax = XMax_;
    YMin = YMin_;
    YMax = YMax_;
    XSize = XSize_;
    YSize = YSize_;
    dy = (YMax_ - YMin_)/YSize_;
    dx = (XMax_ - XMin_)/XSize_;
    values = new TStats* [XSize];
    for (int i = 0; i< XSize; i++)
    {
        values[i] = new TStats[YSize];
       for (int j = 0; j< YSize; j++)
       {
         values[i][j].Clear();
       }
    }
};
THist::~THist()
{
    for (int i = 0; i< XSize; i++)
        delete [] values[i];
    delete [] values;
};
int THist::Add(double x, double y, double value)
{
    if(x >= XMin&& x<XMax && y >= YMin && y<YMax)
    {
        values[int(floor((x - XMin)/dx))][int(floor((y - YMin)/dy))].Add(value);
        return 1;
    }
    else
        return 0;
};
int THist::SaveToFileSum(char* FN)
{
    ofstream F(FN);
    for (int i = 0; i< XSize; i++)
    {
        for (int j = 0; j< YSize; j++)
        {
            F << values[i][j].Sum << '\t';
        }
        F << endl;
    }
    F.close();
    return 0;
};

int THist::SaveToFileCount(char* FN)
{
    ofstream F(FN);
    for (int i = 0; i< XSize; i++)
    {
        for (int j = 0; j< YSize; j++)
        {
            F << values[i][j].Count << '\t';
        }
        F << endl;
    }
    F.close();
    return 0;
};

int THist::SaveToFileMeanStd(char* FN)
{
    ofstream F(FN);
    double cur = XMin - dx/2.0;
    for (int i = 0; i< XSize; i++)
    {
        cur +=  dx;
        for (int j = 0; j< YSize; j++)
        {
            F << cur << '\t' << values[i][j].Mean() << '\t' << sqrt(values[i][j].StdDev());
        }
        F << endl;
    }
    F.close();
    return 0;
};



//-----------------------------------------THist1D---------------------------------------------------------
THist1D::THist1D(double XMin_, double XMax_, int XSize_)
{
    XMin = XMin_;
    XMax = XMax_;
    XSize = XSize_;
    dx = (XMax_ - XMin_)/XSize_;
    values = new TStats[XSize];
    for (int i = 0; i< XSize; i++)
    {
         values[i].Clear();
    }
};
THist1D::~THist1D()
{
    delete [] values;
};

int THist1D::Add(double x, double value)
{
    if(x >= XMin&& x<XMax )
    {
        values[int(floor((x - XMin)/dx))].Add(value);
        return 1;
    }
    else
        return 0;
};


//------------------------------------------TCube-------------------------------------------
TCube::TCube(double XMin_, double XMax_, double YMin_, double YMax_,  double ZMin_, double ZMax_, int XSize_, int YSize_, int ZSize_)
{
    ZMin = ZMin_;
    ZMax = ZMax_;
    ZSize = ZSize_;
    dz = (ZMax_ - ZMin_)/ZSize_;
    values = new pTHist[ZSize_];
    for(int i = 0; i < ZSize_; i++)
     values[i] = new THist( XMin_,  XMax_,  YMin_,  YMax_,  XSize_,  YSize_);
};

TCube::TCube()
{
    delete [] values;
};

int TCube::Add(double x, double y, double z, double value)
{
    if(z >= ZMin&& z<ZMax)
    {
        return values[int(floor((z - ZMin)/dz))]->Add(x, y, value);
    }
    else
        return 0;
};
int TCube::SaveToFileSum(char* FN)
{
    char FNew[40];
    for (int i = 0; i< ZSize; i++)
    {
        sprintf(FNew, "%s%d.txt", FN, i);
        values[i]->SaveToFileSum(FNew);

    }
};
int TCube::SaveToFileCount(char* FN)
{
    char FNew[40];
    for (int i = 0; i< ZSize; i++)
    {
        sprintf(FNew, "%s%d.txt", FN, i);
        values[i]->SaveToFileCount(FNew);
    }
};
int TCube::SaveToFileMeanStd(char* FN)
{
    char FNew[40];
    for (int i = 0; i< ZSize; i++)
    {
        sprintf(FNew, "%s%d.txt", FN, i);
        values[i]->SaveToFileMeanStd(FNew);

    }
};


#endif
